package eu.dnetlib.repo.manager.controllers;

import eu.dnetlib.enabling.datasources.common.AggregationInfo;
import eu.dnetlib.repo.manager.domain.BrokerSummary;
import eu.dnetlib.repo.manager.domain.CollectionMonitorSummary;
import eu.dnetlib.repo.manager.domain.RepositorySummaryInfo;
import eu.dnetlib.repo.manager.domain.UsageSummary;
import eu.dnetlib.repo.manager.exception.BrokerException;
import eu.dnetlib.repo.manager.exception.RepositoryServiceException;
import eu.dnetlib.repo.manager.service.*;
import io.swagger.annotations.Api;
import org.json.JSONException;
import org.mitre.openid.connect.model.OIDCAuthenticationToken;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(value = "/dashboard")
@Api(description = "Dashboard API", tags = {"dashboard"})
public class DashboardController {

    @Autowired
    private DashboardService dashboardService;

    @Autowired
    private RepositoryService repositoryService;

    @Autowired
    private AggregationService aggregationService;

    @Autowired
    private BrokerService brokerService;

    @Autowired
    private PiWikService piWikService;

    @RequestMapping(value = "/getRepositoriesSummary/{page}/{size}", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    @PreAuthorize("hasAuthority('REGISTERED_USER')")
    public List<RepositorySummaryInfo> getRepositoriesSummaryInfo(
            @PathVariable("page") String page,
            @PathVariable("size") String size) throws JSONException {
        return dashboardService.getRepositoriesSummaryInfo(((OIDCAuthenticationToken) SecurityContextHolder.getContext().getAuthentication()).getUserInfo().getEmail(), page, size);
    }

    @RequestMapping(value = "/collectionMonitorSummary/{repoId}", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    @PreAuthorize("hasAuthority('REGISTERED_USER')")
    public CollectionMonitorSummary getCollectionMonitorSummary(
            @PathVariable("repoId") String repoId,
            @RequestParam(name = "size", required = false, defaultValue = "20") int size) throws JSONException {

        // Get the "aggregationInfo" for the first <number of> requested aggregations.
        List<AggregationInfo> aggregationInfoList = aggregationService.getRepositoryAggregations(repoId, 0, size);

        CollectionMonitorSummary collectionMonitorSummary = new CollectionMonitorSummary();
        collectionMonitorSummary.setAggregationInfo(aggregationInfoList);

        // Search for the last indexed version, from the beginning of the list.

        // TODO - WARNING! This is very slow, since each time it requests the next 50, it re-requests the whole list from the "/ds/aggregationhistory/" DSM-endpoint.
        // inside "AggregationServiceImpl.getRepositoryAggregations()". Then it splits the list in a chunk of 50 and then iterate though it here..
        // Since we already request the whole list each time anyway, just take that use it here!!

        size = 0;
        do {
            aggregationInfoList = aggregationService.getRepositoryAggregations(repoId, size, size + 50);
            for (AggregationInfo aggregationInfo : aggregationInfoList) {
                if (aggregationInfo.isIndexedVersion()) {
                    collectionMonitorSummary.setLastIndexedVersion(aggregationInfo);
                    break;
                }
            }
            size += 50;
        } while (aggregationInfoList.size() != 0 && collectionMonitorSummary.getLastIndexedVersion() == null);

        return collectionMonitorSummary;
    }

    @RequestMapping(value = "/usageSummary/{repoId}", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    @PreAuthorize("hasAuthority('REGISTERED_USER')")
    public UsageSummary getUsageSummary(
            @PathVariable("repoId") String repoId
    ) throws RepositoryServiceException {
        return new UsageSummary(repositoryService.getMetricsInfoForRepository(repoId), piWikService.getPiwikSiteForRepo(repoId));
    }

    @RequestMapping(value = "/brokerSummary/{ds_name}", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    @PreAuthorize("hasAuthority('REGISTERED_USER')")
    public BrokerSummary getBrokerSummary(
            @PathVariable("ds_name") String datasourceName) throws BrokerException {
        return new BrokerSummary(brokerService.getSimpleSubscriptionsOfUser(((OIDCAuthenticationToken) SecurityContextHolder.getContext().getAuthentication()).getUserInfo().getEmail()), brokerService.getTopicsForDatasource(datasourceName));
    }

}
